import { __assign } from "tslib";
import { CKBJson } from '@antv/knowledge';
import { deepMix } from '@antv/util';
import { initChartRules, DesignRules } from '../rules';
import _get from 'lodash/get';
import { getChartTypeSpec } from './spec-mapping';
import { customChartType } from '../custom-plot';
function scoreRules(chartType, dataProps, options) {
    var showLog = options === null || options === void 0 ? void 0 : options.showLog;
    var purpose = options ? options.purpose : '';
    var preferences = options ? options.preferences : undefined;
    var chartRuleConfigs = (options === null || options === void 0 ? void 0 : options.chartRuleConfigs) || {};
    var ChartRules = initChartRules(options === null || options === void 0 ? void 0 : options.chartKnowledge);
    // for log
    var record = [];
    var hardScore = 1;
    ChartRules.filter(function (r) {
        return r.hardOrSoft === 'HARD' && r.specChartTypes.includes(chartType) && !_get(chartRuleConfigs, r.id + ".off");
    }).forEach(function (hr) {
        var customConfigs = _get(chartRuleConfigs, "" + hr.id) || {};
        var score = hr.check(__assign({ dataProps: dataProps, chartType: chartType, purpose: purpose, preferences: preferences }, customConfigs));
        hardScore *= score;
        record.push({ name: hr.id, score: score, hard: true });
    });
    /**
     * TODO
     * new soft rules score method
     */
    var softScore = 0;
    // let softFullScore = 0;
    ChartRules.filter(function (r) {
        return r.hardOrSoft === 'SOFT' && r.specChartTypes.includes(chartType) && !_get(chartRuleConfigs, r.id + ".off");
    }).forEach(function (sr) {
        var customConfigs = _get(chartRuleConfigs, "" + sr.id) || {};
        // const weight = _get(chartRuleConfigs, `${sr.id}.weight`) || sr.weight;
        // softFullScore += weight * 1;
        var score = sr.check(__assign({ dataProps: dataProps, chartType: chartType, purpose: purpose, preferences: preferences }, customConfigs));
        softScore += score;
        record.push({ name: sr.id, score: score, hard: false });
    });
    // const score = hardScore * 100 * (softFullScore ? softScore / softFullScore : 0);
    var score = hardScore * (1 + softScore);
    if (showLog)
        console.log('💯score: ', score, '=', hardScore, '* (1 +', softScore, ') ;charttype: ', chartType);
    if (showLog)
        console.log(record);
    return score;
}
function applyDesignRules(chartType, dataProps, chartTypeSpec) {
    var toCheckRules = DesignRules.filter(function (rule) { return rule.domain.indexOf(chartType) !== -1; });
    var encodingSpec = toCheckRules.reduce(function (lastSpec, rule) {
        var relatedSpec = rule.optimizer(dataProps, chartTypeSpec);
        return deepMix(lastSpec, relatedSpec);
    }, {});
    return encodingSpec;
}
/**
 * @public
 */
export function dataPropsToAdvices(dataProps, options, dataset) {
    var enableRefine = (options === null || options === void 0 ? void 0 : options.refine) === undefined ? true : options.refine;
    var showLog = options === null || options === void 0 ? void 0 : options.showLog;
    var ChartWIKI = (options === null || options === void 0 ? void 0 : options.chartKnowledge) ? options.chartKnowledge : CKBJson('en-US', true);
    var CHART_ID_OPTIONS = Object.keys(ChartWIKI);
    // score every
    var list = CHART_ID_OPTIONS.map(function (t) {
        // step 1: analyze score by rule
        var score = scoreRules(t, dataProps, options);
        if (score <= 0)
            return { type: t, spec: null, score: score };
        // step 2: field mapping to spec encoding
        var chartTypeSpec = getChartTypeSpec(t, dataProps, dataset);
        // FIXME kpi_panel and table spec 暂时可以为 null, 之后随需求增加 cfg
        if (!customChartType.includes(t) && !chartTypeSpec)
            return { type: t, spec: null, score: 0 };
        // step 3: apply design rules
        if (chartTypeSpec && enableRefine) {
            var encodingSpecs = applyDesignRules(t, dataProps, chartTypeSpec);
            deepMix(chartTypeSpec.encoding, encodingSpecs);
            // return { type: t, spec: chartTypeSpec, score }
        }
        return { type: t, spec: chartTypeSpec, score: score };
    });
    function compareAdvices(chart1, chart2) {
        if (chart1.score < chart2.score) {
            return 1;
        }
        else if (chart1.score > chart2.score) {
            return -1;
        }
        else {
            return 0;
        }
    }
    // filter and sorter
    var resultList = list.filter(function (e) { return e.score > 0; }).sort(compareAdvices);
    if (showLog)
        console.log('🍒🍒🍒🍒🍒🍒 resultList 🍒🍒🍒🍒🍒🍒');
    if (showLog)
        console.log(resultList);
    return resultList;
}
