import { __awaiter, __generator, __read, __values } from "tslib";
import { merge, set, get } from 'lodash';
import { isEqual, pick } from '@antv/util';
import { ConfigPanel } from './config-panel';
import { Toolbar } from './toolbar';
import { AutoPlot } from './auto-plot';
import { DummyPlot } from './dummy-plot';
import { customChartType } from '../custom-plot';
import { MockPanel } from './mock-panel';
import { createLayer, DEFAULT_FEEDBACK } from '../util';
import { intl, setLanguage } from '../i18n';
var CACHES = new Map();
/**
 * clean the instance binding with the destroyed container
 */
function CheckAndClean() {
    var e_1, _a;
    try {
        for (var _b = __values(CACHES.entries()), _c = _b.next(); !_c.done; _c = _b.next()) {
            var _d = __read(_c.value, 2), container = _d[0], chart = _d[1];
            if (!document.contains(container)) {
                chart.destroy();
                CACHES.delete(container);
            }
        }
    }
    catch (e_1_1) { e_1 = { error: e_1_1 }; }
    finally {
        try {
            if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
        }
        finally { if (e_1) throw e_1.error; }
    }
    window.requestAnimationFrame(CheckAndClean);
}
window.requestAnimationFrame(CheckAndClean);
export function addCanvas(layer, _a) {
    var title = _a.title, description = _a.description;
    layer.style.display = 'flex';
    layer.style.flexDirection = 'column';
    layer.style.padding = '24px';
    if (title) {
        var titleDiv = document.createElement('div');
        titleDiv.style.flex = 'none';
        titleDiv.style.fontSize = '28px';
        titleDiv.style.paddingBottom = '16px';
        titleDiv.innerHTML = title;
        layer.appendChild(titleDiv);
    }
    if (description) {
        var descriptionDiv = document.createElement('div');
        descriptionDiv.style.flex = 'none';
        descriptionDiv.style.fontSize = '13px';
        descriptionDiv.style.color = '#8e8e8e';
        descriptionDiv.innerHTML = description;
        descriptionDiv.style.paddingBottom = '16px';
        layer.appendChild(descriptionDiv);
    }
    var canvas = document.createElement('div');
    canvas.style.flex = '1';
    layer.appendChild(canvas);
    return canvas;
}
// default don't advice table in autoChart
var defaultOptions = {
    chartRuleConfigs: {
        'all-can-be-table': {
            weight: 0,
        },
    },
};
/**
 * @public
 */
var AutoChart = /** @class */ (function () {
    function AutoChart(container) {
        this.rendered = false;
        this.options = {};
        this.isMocked = false;
        this.container = container;
        this.noDataLayer = createLayer(container, 'no-data-layer');
    }
    AutoChart.create = function (container, data, options) {
        return __awaiter(this, void 0, void 0, function () {
            var containerDom, dataP, inst;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        containerDom = typeof container === 'string' ? document.getElementById(container) : container;
                        if (!containerDom)
                            throw new Error('please make sure canvas container is not null');
                        if (!Array.isArray(data)) return [3 /*break*/, 1];
                        dataP = data;
                        return [3 /*break*/, 4];
                    case 1:
                        if (!(data instanceof Promise)) return [3 /*break*/, 3];
                        return [4 /*yield*/, data];
                    case 2:
                        dataP = _a.sent();
                        return [3 /*break*/, 4];
                    case 3: throw new TypeError('data type is error');
                    case 4:
                        if (!dataP || !Array.isArray(dataP)) {
                            throw new TypeError('data type is error');
                        }
                        if (CACHES.get(containerDom)) {
                            inst = CACHES.get(containerDom);
                            //  return directly while data and config is not change
                            if (isEqual(inst.options, options) && isEqual(inst.data, data))
                                return [2 /*return*/, inst];
                        }
                        else {
                            inst = new AutoChart(containerDom);
                            CACHES.set(containerDom, inst);
                        }
                        return [4 /*yield*/, inst.setup(dataP, options)];
                    case 5: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    AutoChart.prototype.setup = function (data, options) {
        return __awaiter(this, void 0, void 0, function () {
            var _a, fields, development, noDataContent, language;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (this.rendered)
                            this.destroy();
                        this.rendered = true;
                        // this.isMocked = false;
                        this.options = merge({}, defaultOptions, options);
                        _a = this.options, fields = _a.fields, development = _a.development, noDataContent = _a.noDataContent, language = _a.language;
                        if (!this.noDataLayer)
                            this.noDataLayer = createLayer(this.container, 'no-data-layer');
                        this.noDataContent = noDataContent || DEFAULT_FEEDBACK(intl.get('No Recommendation'));
                        this.data = fields && fields.length > 0 ? data.map(function (item) { return pick(item, fields); }) : data;
                        this.development =
                            (development === undefined && process.env.NODE_ENV === 'development') ||
                                (development !== undefined && development === true);
                        setLanguage(language);
                        return [4 /*yield*/, this.render()];
                    case 1:
                        _b.sent();
                        return [2 /*return*/, this];
                }
            });
        });
    };
    AutoChart.prototype.render = function () {
        return __awaiter(this, void 0, void 0, function () {
            var _a, options, container, development, noDataContent, theme, toolbar, purpose, preferences, chartRuleConfigs, refine, config, mockPanel, result, chartCanvas, colorField_1, oldAdvices, oldIndex;
            var _this = this;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = this, options = _a.options, container = _a.container, development = _a.development, noDataContent = _a.noDataContent;
                        theme = options.theme, toolbar = options.toolbar, purpose = options.purpose, preferences = options.preferences, chartRuleConfigs = options.chartRuleConfigs, refine = options.refine;
                        config = options.config;
                        if (!(this.data && this.data.length === 0)) return [3 /*break*/, 3];
                        if (!development) return [3 /*break*/, 2];
                        mockPanel = new MockPanel(container);
                        this.mockPanel = mockPanel;
                        return [4 /*yield*/, mockPanel.ps];
                    case 1:
                        result = _b.sent();
                        this.data = result.data;
                        config = result.config;
                        this.mockPanel.destroy();
                        return [3 /*break*/, 3];
                    case 2:
                        noDataContent(this.noDataLayer);
                        return [2 /*return*/];
                    case 3:
                        this.canvasLayer = createLayer(this.container, 'canvas-layer');
                        this.canvasLayer.style.pointerEvents = 'auto';
                        chartCanvas = addCanvas(this.canvasLayer, options);
                        if (config) {
                            if (typeof config.type !== 'string')
                                throw new Error('please set the plotType');
                            if (config.type === 'Pie' && (config === null || config === void 0 ? void 0 : config.configs)) {
                                colorField_1 = get(config.configs, 'colorField');
                                if (colorField_1 && !get(config.configs, 'statistic.title.formatter')) {
                                    set(config.configs, 'statistic.title.formatter', function (datum) {
                                        if (datum) {
                                            return datum[colorField_1];
                                        }
                                        return intl.get('Total');
                                    });
                                }
                            }
                            this.plot = new DummyPlot(chartCanvas, this.data, config);
                        }
                        else {
                            oldAdvices = [];
                            oldIndex = 0;
                            if (this.plot instanceof AutoPlot) {
                                oldAdvices = this.plot.advices;
                                oldIndex = this.plot.current;
                            }
                            this.plot = new AutoPlot(chartCanvas, this.data, { theme: theme, purpose: purpose, preferences: preferences, chartRuleConfigs: chartRuleConfigs, refine: refine }, oldAdvices, oldIndex);
                            if (toolbar && this.plot.advices.length > 0) {
                                this.toolbar = new Toolbar(this.plot, this.container);
                            }
                        }
                        // plot instance is async render due to import g2plot using webpackChunkName
                        // make this judge(this.plot.plot) in next event loop
                        setTimeout(function () {
                            var _a, _b;
                            if (development && ((_a = _this.plot) === null || _a === void 0 ? void 0 : _a.plot)) {
                                // configPanel not supported kpi_panel and table temporary
                                if (!customChartType.includes(((_b = _this.plot) === null || _b === void 0 ? void 0 : _b.type) || '')) {
                                    _this.configPanel = new ConfigPanel(_this.plot, _this.isMocked, _this.container, _this.options.language);
                                }
                            }
                        });
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * 清除上次渲染遗留的元素
     */
    AutoChart.prototype.destroy = function () {
        if (this.container.contains(this.noDataLayer)) {
            this.container.removeChild(this.noDataLayer);
        }
        if (this.configPanel)
            this.configPanel.destroy();
        if (this.toolbar)
            this.toolbar.destroy();
        if (this.plot)
            this.plot.destroy();
        if (this.canvasLayer) {
            this.container.removeChild(this.canvasLayer);
            this.canvasLayer = undefined;
        }
        if (this.mockPanel)
            this.mockPanel.destroy();
    };
    /**
     * get plot detail
     */
    AutoChart.prototype.getPlot = function () {
        return this.plot;
    };
    return AutoChart;
}());
export { AutoChart };
