import { __assign, __extends } from "tslib";
import { set, get } from 'lodash';
import { dataToAdvices, adviceToLibConfig } from '../advice-pipeline';
import EventEmitter from '@antv/event-emitter';
import { uuid, createLayer, DEFAULT_FEEDBACK } from '../util';
import { KPIPlot, Table } from '../custom-plot';
import { intl } from '../i18n';
/**
 * @public
 */
var AutoPlot = /** @class */ (function (_super) {
    __extends(AutoPlot, _super);
    /**
     * Constructor
     * @param container - 图表容器
     * @param data - 图表数据
     * @param options - 图表配置
     * @param oldAdvices - 同一个容器上次渲染的 advice列表
     * @param oldIndex - 同一个容器上次渲染的目标位置
     */
    function AutoPlot(container, data, options, oldAdvices, oldIndex, type) {
        if (options === void 0) { options = {}; }
        if (oldAdvices === void 0) { oldAdvices = []; }
        if (oldIndex === void 0) { oldIndex = 0; }
        var _this = _super.call(this) || this;
        /**
         * 图表Schema列表
         */
        _this.advices = [];
        /**
         * 图表唯一ID
         */
        _this.uuid = uuid();
        /**
         * 当前激活(使用)的图表schema
         */
        _this.current = 0;
        var _a = container.getBoundingClientRect(), width = _a.width, height = _a.height;
        if (!options.preferences || !options.preferences.canvasLayout) {
            options.preferences = {
                canvasLayout: width / height > 1 ? 'landscape' : 'portrait',
            };
        }
        _this.container = container;
        _this.feedbackLayer = createLayer(container, 'feedback-layer');
        // TODO 给 autoChart 增加 fields 参数
        var advices = dataToAdvices(data, options);
        _this.advices = advices;
        _this.options = options;
        _this.feedback = _this.options.feedback || DEFAULT_FEEDBACK(intl.get('No Recommendation'));
        _this.data = data;
        if (_this.advices.length === 0) {
            // throw new Error('推荐不了图表');
            _this.feedback(_this.feedbackLayer);
        }
        else {
            // 如果和上次的结果一样 可以认定为是重复渲染，然后使用上次的index
            if (advices.length === oldAdvices.length && advices.every(function (item, i) { return item.type === advices[i].type; })) {
                _this.render(oldIndex);
            }
            else if (type) {
                var index = 0;
                for (var i = 0; i < advices.length; i++) {
                    if (advices[i].type === type) {
                        index = i;
                    }
                }
                _this.render(index);
            }
            else {
                _this.render(0);
            }
        }
        return _this;
    }
    /**
     * 渲染指定位置的schema
     * @param index - schema 的index
     */
    AutoPlot.prototype.render = function (index) {
        var _this = this;
        var _a = this, advices = _a.advices, options = _a.options, data = _a.data, container = _a.container, current = _a.current, plot = _a.plot;
        if (plot && index === current)
            return;
        this.current = index;
        var type = advices[index].type;
        var currentType = advices[current].type;
        var theme = options.theme;
        var libConfig = adviceToLibConfig(advices[index], 'G2Plot');
        this.type = type;
        if (type === 'kpi_panel') {
            if (plot)
                plot.destroy();
            if (data.length > 0)
                this.plot = new KPIPlot(container, data[0]);
            this.currentConfigs = {};
        }
        else if (type === 'table') {
            if (plot)
                plot.destroy();
            this.plot = new Table(container, data);
            this.currentConfigs = {};
        }
        else {
            if (libConfig) {
                var configs_1 = __assign(__assign({}, libConfig === null || libConfig === void 0 ? void 0 : libConfig.configs), { data: data, theme: theme, autoFit: true });
                if (libConfig.type === 'Pie') {
                    var colorField_1 = get(configs_1, 'colorField');
                    if (colorField_1 && !get(configs_1, 'statistic.title.formatter')) {
                        set(configs_1, 'statistic.title.formatter', function (datum) {
                            if (datum) {
                                return datum[colorField_1];
                            }
                            return intl.get('Total');
                        });
                    }
                }
                this.currentConfigs = configs_1;
                if (plot && type === currentType) {
                    plot.update(configs_1);
                }
                else {
                    if (plot)
                        plot.destroy();
                    import(/* webpackChunkName: "g2plot" */ '@antv/g2plot').then(function (G2Plot) {
                        // @ts-ignore
                        _this.plot = new G2Plot[libConfig.type](container, configs_1);
                        _this.plot.render();
                    });
                }
            }
        }
        // this.plot!.render();
        this.emit('change', [index]);
    };
    AutoPlot.prototype.destroy = function () {
        if (this.plot && !this.plot.destroyed) {
            this.plot.destroy();
            this.container.removeChild(this.feedbackLayer);
            this.plot = undefined;
        }
    };
    return AutoPlot;
}(EventEmitter));
export { AutoPlot };
