import { __assign } from "tslib";
import { CLASS_PREFIX } from '../style';
import { PATH_PREFIX } from './constants';
import { AutoPlot } from './auto-plot';
import { translate, getPosition } from '../util';
import { DevPanel } from './dev-panel';
import { intl, getLanguage } from '../i18n';
var SEND_CONFIGS = '__advisor__.send_configs';
var CONFIGS_CHANGE = '__advisor__.configs_change';
var ConfigPanel = /** @class */ (function () {
    function ConfigPanel(plotInst, needCopyData, container, language) {
        var _this = this;
        /**
         * config panel 的按钮
         */
        this.trigger = document.createElement('div');
        /**
         * iframe
         */
        this.iframe = document.createElement('iframe');
        /**
         * 是否以销毁
         */
        this.destroyed = false;
        this.messageHandler = function (e) {
            var _a = _this.plotInst, uuid = _a.uuid, plot = _a.plot;
            if (e.data && e.data.type === CONFIGS_CHANGE && e.data.uuid === uuid) {
                if (plot) {
                    plot.update(e.data.configs);
                    plot.render();
                }
            }
        };
        this.mouseEnterHandler = function () {
            _this.trigger.style.display = 'block';
        };
        this.mouseLeaveHandler = function () {
            _this.trigger.style.display = 'none';
        };
        this.plotInst = plotInst;
        this.needCopyData = needCopyData;
        this.chartContainer = container;
        this.language = language || getLanguage();
        container.addEventListener('mouseenter', this.mouseEnterHandler);
        container.addEventListener('mouseleave', this.mouseLeaveHandler);
        this.initIframe();
        this.initTrigger();
        container.appendChild(this.trigger);
        this.changeType();
    }
    ConfigPanel.prototype.initTrigger = function () {
        var _this = this;
        var _a = this, trigger = _a.trigger, iframe = _a.iframe;
        trigger.innerHTML = '<img src="https://gw.alipayobjects.com/zos/antfincdn/zKMUjshkQt/config.png" />';
        trigger.className = CLASS_PREFIX + "dev_btn " + CLASS_PREFIX + "config_btn";
        trigger.addEventListener('click', function () {
            if (!_this.panel) {
                _this.panel = new DevPanel(__assign(__assign({ title: intl.get('Chart Config', _this.language), height: 534, width: 340 }, getPosition(_this.chartContainer)), { visible: true, content: iframe }));
            }
            else {
                _this.panel.toggle();
            }
        });
    };
    ConfigPanel.prototype.initIframe = function () {
        var _this = this;
        var _a = this, iframe = _a.iframe, plotInst = _a.plotInst;
        iframe.addEventListener('load', function () {
            var currentConfigs = plotInst.currentConfigs, uuid = plotInst.uuid;
            var message = {
                uuid: uuid,
                configs: JSON.parse(JSON.stringify(currentConfigs)),
                type: SEND_CONFIGS,
            };
            iframe.contentWindow.postMessage(message, '*');
        });
        window.addEventListener('message', this.messageHandler);
        if (plotInst instanceof AutoPlot) {
            plotInst.on('change', function () {
                _this.changeType();
            });
        }
    };
    /**
     * 销毁实例
     */
    ConfigPanel.prototype.destroy = function () {
        var _a = this, chartContainer = _a.chartContainer, trigger = _a.trigger, panel = _a.panel, destroyed = _a.destroyed;
        if (destroyed)
            return;
        if (panel)
            panel.destroy();
        chartContainer.removeChild(trigger);
        chartContainer.removeEventListener('mouseenter', this.mouseEnterHandler);
        chartContainer.removeEventListener('mouseleave', this.mouseLeaveHandler);
        window.removeEventListener('message', this.messageHandler);
        this.destroyed = true;
    };
    /**
     * 响应面板类型变更
     */
    ConfigPanel.prototype.changeType = function () {
        var type = this.plotInst.type;
        if (type)
            this.iframe.src = PATH_PREFIX + "/config-panel.html?type=" + translate(type) + "&cancopydata=" + this.needCopyData + "&lang=" + this.language;
    };
    return ConfigPanel;
}());
export { ConfigPanel };
