import { __assign } from "tslib";
import { PATH_PREFIX } from './constants';
import { CLASS_PREFIX } from '../style';
import { uuid, getPosition } from '../util';
import { DevPanel } from './dev-panel';
import { intl, getLanguage } from '../i18n';
var MOCK_CHART = '__advisor__.mock_chart';
var CALLBACKS = new Map();
window.addEventListener('message', function (e) {
    if (e.data && e.data.type === MOCK_CHART) {
        var _a = e.data, id = _a.id, error = _a.error, result = _a.result;
        var cache = CALLBACKS.get(id);
        if (cache) {
            var reject = cache.reject, resolve = cache.resolve;
            if (error) {
                reject(error);
            }
            else {
                resolve(result);
            }
        }
    }
});
var MockPanel = /** @class */ (function () {
    function MockPanel(container) {
        var _this = this;
        /**
         * config panel 的按钮
         */
        this.trigger = document.createElement('div');
        this.destroyed = false;
        var trigger = this.trigger;
        this.chartContainer = container;
        var id = uuid();
        this.ps = new Promise(function (resolve, reject) {
            CALLBACKS.set(id, { resolve: resolve, reject: reject });
        });
        trigger.className = CLASS_PREFIX + "mock_guide";
        trigger.innerHTML = "\n      <div style=\"margin-bottom: 16px;\">\n        <img src=\"https://gw.alipayobjects.com/zos/basement_prod/9a59280d-8f23-4234-b5cf-02956a91b6ff.svg\" />\n      </div>\n      <div>" + intl.get('No Data') + "</div>\n      <div class=\"" + CLASS_PREFIX + "mock_guide_button\">" + intl.get('Initialize') + "</div>\n    ";
        trigger.addEventListener('click', function () {
            if (!_this.panel) {
                _this.panel = new DevPanel(__assign(__assign({ title: intl.get('Initialize'), height: 616, width: 600 }, getPosition(container)), { visible: true, content: "<iframe src=\"" + PATH_PREFIX + "/mock-panel.html?id=" + id + "&lang=" + getLanguage() + "\"></iframe>" }));
            }
            else {
                _this.panel.toggle();
            }
        });
        container.appendChild(trigger);
    }
    /**
     * 销毁实例
     */
    MockPanel.prototype.destroy = function () {
        var _a = this, chartContainer = _a.chartContainer, trigger = _a.trigger, panel = _a.panel, destroyed = _a.destroyed;
        if (destroyed)
            return;
        if (panel)
            panel.destroy();
        chartContainer.removeChild(trigger);
        this.destroyed = true;
    };
    return MockPanel;
}());
export { MockPanel };
