import { __awaiter, __generator, __read, __spread } from "tslib";
import { Insight as VisualInsight } from 'visual-insights';
import { type as typeAnalyze, isUnique, isTime, isInterval, isOrdinal, } from '@antv/dw-analyzer';
import { getInsightSpaces } from '../fromVisualInsights';
export function pearsonCorr(d1, d2) {
    var _a;
    var min = Math.min, pow = Math.pow, sqrt = Math.sqrt;
    var add = function (a, b) { return a + b; };
    var n = min(d1.length, d2.length);
    if (n === 0) {
        return 0;
    }
    _a = __read([d1.slice(0, n), d2.slice(0, n)], 2), d1 = _a[0], d2 = _a[1];
    var _b = __read([d1, d2].map(function (l) { return l.reduce(add); }), 2), sum1 = _b[0], sum2 = _b[1];
    var _c = __read([d1, d2].map(function (l) { return l.reduce(function (a, b) { return a + pow(b, 2); }, 0); }), 2), pow1 = _c[0], pow2 = _c[1];
    var mulSum = d1.map(function (n, i) { return n * d2[i]; }).reduce(add);
    var dense = sqrt((pow1 - pow(sum1, 2) / n) * (pow2 - pow(sum2, 2) / n));
    if (dense === 0) {
        return 0;
    }
    return (mulSum - (sum1 * sum2) / n) / dense;
}
export function rowDataToColumnFrame(rows) {
    if (!rows || rows.length === 0) {
        return {
            columnProps: [{ title: '', type: null }],
            columns: [[]],
        };
    }
    var titles = Object.keys(rows[0]);
    var columnProps = [];
    var columns = [];
    titles.forEach(function (title) {
        var column = rows.map(function (row) { return row[title]; });
        var anaResult = typeAnalyze(column);
        columnProps.push({
            title: title,
            type: anaResult.recommendation,
            isUnique: isUnique(anaResult),
            isTime: isTime(anaResult),
            isInterval: isInterval(anaResult),
            isOrdinal: isOrdinal(anaResult),
            fieldInfo: anaResult,
        });
        columns.push(column);
    });
    return {
        columnProps: columnProps,
        columns: columns,
    };
}
export function columnsToRowData(columns, titles) {
    if (!columns.length || !titles.length || columns.length !== titles.length) {
        return [];
    }
    var rowLength = columns[0].length;
    // Lengths of columns should be same.
    for (var k = 0; k < columns.length; k++) {
        if (columns[k].length !== rowLength) {
            return [];
        }
    }
    var result = [];
    for (var i = 0; i < rowLength; i++) {
        var row = {};
        for (var j = 0; j < columns.length; j++) {
            row[titles[j]] = columns[j][i];
        }
        result.push(row);
    }
    return result;
}
export function insightSpaceToInsight(space) {
    var dimensions = space.dimensions, measures = space.measures, significance = space.significance, type = space.type, score = space.score, description = space.description;
    var _insightProps = {};
    var _type = type ? type : 'SomeInsight';
    var _fields = [];
    if (dimensions && dimensions.length) {
        _fields = __spread(_fields, dimensions);
        _insightProps.dimensions = dimensions;
    }
    if (measures && measures.length) {
        _fields = __spread(_fields, measures);
        _insightProps.measures = measures;
    }
    var _score = score ? score : significance;
    if (_score) {
        _insightProps.score = _score;
    }
    if (description) {
        _insightProps.detail = description;
    }
    var insight = {
        type: _type,
        fields: _fields,
    };
    if (!(Object.keys(_insightProps).length === 0 && _insightProps.constructor === Object)) {
        insight.insightProps = _insightProps;
    }
    return insight;
}
export function visualInsightWorkerToAVAWorker(viWorkerId) {
    var AVAWorker = function (data) {
        return __awaiter(this, void 0, void 0, function () {
            var workerCollection, spaces, insights;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        workerCollection = VisualInsight.IntentionWorkerCollection.init({ withDefaultIWorkers: false });
                        workerCollection.enable(viWorkerId, true);
                        return [4 /*yield*/, getInsightSpaces({
                                dataSource: data,
                                collection: workerCollection,
                                enableUniqueFields: false,
                            })];
                    case 1:
                        spaces = _a.sent();
                        insights = [];
                        spaces.forEach(function (space) {
                            var insight = insightSpaceToInsight(space);
                            insights.push(insight);
                        });
                        return [2 /*return*/, insights];
                }
            });
        });
    };
    return AVAWorker;
}
export function isMonotonicInc(array) {
    for (var i = 1; i < array.length; i++) {
        if (array[i - 1] >= array[i]) {
            return false;
        }
    }
    return true;
}
export function isMonotonicDec(array) {
    for (var i = 1; i < array.length; i++) {
        if (array[i - 1] <= array[i]) {
            return false;
        }
    }
    return true;
}
export function normalizeArray(array) {
    var max = Math.max.apply(Math, __spread(array));
    var min = Math.min.apply(Math, __spread(array));
    return array.map(function (val) {
        return (val - min) / (max - min);
    });
}
export function outliersFilters(dataProps) {
    var threshold = 3;
    return dataProps.map(function (dp) {
        if (dp.recommendation === 'float' || dp.recommendation === 'integer') {
            var q1 = dp.percentile25;
            var q3 = dp.percentile75;
            var iqr = Math.abs(q3 - q1);
            var lowThreshold_1 = q1 - threshold * iqr;
            var highThreshold_1 = q3 + threshold * iqr;
            return function (v) { return v < lowThreshold_1 || v > highThreshold_1; };
        }
        return function (_) { return false; };
    });
}
