import { __values } from "tslib";
import { Rule } from './concepts/rule';
import { isUndefined } from 'lodash';
import { CKBJson, } from '@antv/knowledge';
function compare(f1, f2) {
    if (f1.distinct < f2.distinct) {
        return 1;
    }
    else if (f1.distinct > f2.distinct) {
        return -1;
    }
    else {
        return 0;
    }
}
function hasSubset(array1, array2) {
    return array2.every(function (e) { return array1.includes(e); });
}
function intersects(array1, array2) {
    return array2.some(function (e) { return array1.includes(e); });
}
function verifyDataProps(dataPre, dataProps) {
    var e_1, _a;
    var fieldsLOMs = dataProps.map(function (info) {
        return info.levelOfMeasurements;
    });
    if (fieldsLOMs) {
        var lomCount = 0;
        try {
            for (var fieldsLOMs_1 = __values(fieldsLOMs), fieldsLOMs_1_1 = fieldsLOMs_1.next(); !fieldsLOMs_1_1.done; fieldsLOMs_1_1 = fieldsLOMs_1.next()) {
                var fieldLOM = fieldsLOMs_1_1.value;
                if (fieldLOM && intersects(fieldLOM, dataPre.fieldConditions)) {
                    lomCount += 1;
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (fieldsLOMs_1_1 && !fieldsLOMs_1_1.done && (_a = fieldsLOMs_1.return)) _a.call(fieldsLOMs_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
        if (lomCount >= dataPre.minQty && (lomCount <= dataPre.maxQty || dataPre.maxQty === '*')) {
            return true;
        }
    }
    return false;
}
/**
 * @public
 */
export var initChartRules = function (chartWiki) {
    var Wiki = chartWiki ? chartWiki : CKBJson('en-US', true);
    var allChartTypes = Object.keys(Wiki);
    return [
        // Data must satisfy the data prerequisites
        new Rule('data-check', 'HARD', allChartTypes, 1.0, function (args) {
            var e_2, _a;
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && chartType && Wiki[chartType]) {
                result = 1;
                var dataPres = Wiki[chartType].dataPres || [];
                try {
                    for (var dataPres_1 = __values(dataPres), dataPres_1_1 = dataPres_1.next(); !dataPres_1_1.done; dataPres_1_1 = dataPres_1.next()) {
                        var dataPre = dataPres_1_1.value;
                        if (!verifyDataProps(dataPre, dataProps)) {
                            result = 0;
                            return result;
                        }
                    }
                }
                catch (e_2_1) { e_2 = { error: e_2_1 }; }
                finally {
                    try {
                        if (dataPres_1_1 && !dataPres_1_1.done && (_a = dataPres_1.return)) _a.call(dataPres_1);
                    }
                    finally { if (e_2) throw e_2.error; }
                }
            }
            return result;
        }),
        // Data must has the min qty of the prerequisite.
        new Rule('data-field-qty', 'HARD', allChartTypes, 1.0, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && chartType && Wiki[chartType]) {
                result = 1;
                var dataPres = Wiki[chartType].dataPres || [];
                var minFieldQty = dataPres.map(function (e) { return e.minQty; }).reduce(function (acc, cv) { return acc + cv; });
                if (dataProps.length) {
                    var fieldQty = dataProps.length;
                    if (fieldQty >= minFieldQty) {
                        result = 1;
                    }
                }
            }
            return result;
        }),
        // No redundant field
        new Rule('no-redundant-field', 'HARD', allChartTypes, 1.0, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && chartType && Wiki[chartType]) {
                var dataPres = Wiki[chartType].dataPres || [];
                var maxFieldQty = dataPres
                    .map(function (e) {
                    if (e.maxQty === '*') {
                        return 99;
                    }
                    return e.maxQty;
                })
                    .reduce(function (acc, cv) { return acc + cv; });
                if (dataProps.length) {
                    var fieldQty = dataProps.length;
                    if (fieldQty <= maxFieldQty) {
                        result = 1;
                    }
                }
            }
            return result;
        }),
        // Choose types that satisfy the purpose, if purpose is defined.
        new Rule('purpose-check', 'HARD', allChartTypes, 1.0, function (args) {
            var result = 0;
            var chartType = args.chartType, purpose = args.purpose;
            // if purpose is not defined
            if (!purpose) {
                result = 1;
                return result;
            }
            if (chartType && Wiki[chartType] && purpose) {
                var purp = Wiki[chartType].purpose || '';
                if (purp.includes(purpose)) {
                    result = 1;
                    return result;
                }
            }
            return result;
        }),
        // Only single row for all columns
        new Rule('aggregation-single-row', 'HARD', allChartTypes.filter(function (i) { return i !== 'table'; }), 1.0, function (args) {
            var result = 0;
            var chartType = args.chartType, dataProps = args.dataProps;
            if (dataProps.every(function (i) { return i.count === 1 && i.levelOfMeasurements.includes('Interval'); })) {
                result = chartType === 'kpi_panel' ? 1 : 0.2;
            }
            else {
                result = chartType === 'kpi_panel' ? 0 : 1;
            }
            return result;
        }),
        // all dataset can be table
        new Rule('all-can-be-table', 'HARD', ['table'], 1.0, function (_a) {
            var weight = _a.weight;
            return isUndefined(weight) ? 1 : weight;
        }),
        // Some charts should has at most N series.
        new Rule('series-qty-limit', 'SOFT', ['pie_chart', 'donut_chart', 'radar_chart', 'rose_chart'], 0.8, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            var limit = args.limit;
            if (!Number.isInteger(limit) || limit <= 0) {
                limit = 6;
                if (chartType === 'pie_chart' || chartType === 'donut_chart' || chartType === 'rose_chart')
                    limit = 6;
                if (chartType === 'radar_chart')
                    limit = 8;
            }
            if (dataProps) {
                var field4Series = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
                var seriesQty = field4Series && field4Series.count ? field4Series.count : 0;
                if (seriesQty >= 2 && seriesQty <= limit) {
                    result = 2 / seriesQty;
                }
            }
            return result;
        }),
        // Bar chart should has proper number of bars or bar groups.
        new Rule('bar-series-qty', 'SOFT', [
            'bar_chart',
            'grouped_bar_chart',
            'stacked_bar_chart',
            'percent_stacked_bar_chart',
            'column_chart',
            'grouped_column_chart',
            'stacked_column_chart',
            'percent_stacked_column_chart',
        ], 0.5, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && chartType) {
                var field4Series = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
                var seriesQty = field4Series && field4Series.count ? field4Series.count : 0;
                // TODO limit for this rule
                if (seriesQty >= 2 && seriesQty <= 20) {
                    result = 1;
                }
                else if (seriesQty > 20) {
                    result = 20 / seriesQty;
                }
            }
            return result;
        }),
        // Data has Time or Ordinal field are good for Line, Area charts.
        new Rule('line-field-time-ordinal', 'SOFT', ['line_chart', 'area_chart', 'stacked_area_chart', 'percent_stacked_area_chart'], 1.0, function (args) {
            var result = 0;
            var dataProps = args.dataProps;
            if (dataProps) {
                var field4TimeOrOrdinal = dataProps.find(function (field) {
                    return intersects(field.levelOfMeasurements, ['Ordinal', 'Time']);
                });
                if (field4TimeOrOrdinal) {
                    result = 1;
                }
            }
            return result;
        }),
        // Landscape or portrait as perferences.
        new Rule('landscape-or-portrait', 'SOFT', [
            'bar_chart',
            'grouped_bar_chart',
            'stacked_bar_chart',
            'percent_stacked_bar_chart',
            'column_chart',
            'grouped_column_chart',
            'stacked_column_chart',
            'percent_stacked_column_chart',
        ], 0.3, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType, preferences = args.preferences;
            if (dataProps && chartType && preferences && preferences.canvasLayout) {
                if (preferences.canvasLayout === 'portrait' &&
                    ['bar_chart', 'grouped_bar_chart', 'stacked_bar_chart', 'percent_stacked_bar_chart'].includes(chartType)) {
                    result = 1;
                }
                else if (preferences.canvasLayout === 'landscape' &&
                    ['column_chart', 'grouped_column_chart', 'stacked_column_chart', 'percent_stacked_column_chart'].includes(chartType)) {
                    result = 1;
                }
            }
            return result;
        }),
        // Difference should be big enough for pie sectors.
        new Rule('diff-pie-sector', 'SOFT', ['pie_chart', 'donut_chart'], 0.5, function (args) {
            var result = 0;
            var dataProps = args.dataProps;
            if (dataProps) {
                var intervalField = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
                if (intervalField && intervalField.sum && intervalField.samples) {
                    var sum = intervalField.sum;
                    var scale_1 = 1 / sum;
                    var scaledSamples = intervalField.samples.map(function (v) { return v * scale_1; });
                    var scaledProduct = scaledSamples.reduce(function (a, c) { return a * c; });
                    var count = intervalField.samples.length;
                    var maxProduct = Math.pow(1 / count, count);
                    result = Math.abs(maxProduct - Math.abs(scaledProduct)) / maxProduct;
                }
            }
            return result;
        }),
        // Single (Basic) and Multi (Stacked, Grouped,...) charts should be optimizedly recommended by nominal enums combinatorial numbers.
        new Rule('nominal-enum-combinatorial', 'SOFT', allChartTypes, 1.0, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && allChartTypes) {
                var nominalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
                if (nominalFields.length >= 2) {
                    var sortedNominals = nominalFields.sort(compare);
                    var f1 = sortedNominals[0];
                    var f2 = sortedNominals[1];
                    if (f1.distinct === f1.count) {
                        if (['bar_chart', 'column_chart'].includes(chartType)) {
                            result = 1;
                        }
                    }
                    if (f1.count && f1.distinct && f2.distinct && f1.count > f1.distinct) {
                        var typeOptions = [
                            'grouped_bar_chart',
                            'grouped_column_chart',
                            'stacked_bar_chart',
                            'stacked_column_chart',
                        ];
                        if (typeOptions.includes(chartType)) {
                            result = 1;
                        }
                    }
                }
            }
            return result;
        }),
        // Avoid too many series
        new Rule('limit-series', 'SOFT', allChartTypes, 1.0, function (args) {
            var result = 0;
            var dataProps = args.dataProps, chartType = args.chartType;
            if (dataProps && allChartTypes) {
                var nominalOrOrdinalFields = dataProps.filter(function (field) {
                    return intersects(field.levelOfMeasurements, ['Nominal', 'Ordinal']);
                });
                if (nominalOrOrdinalFields.length >= 2) {
                    var sortedFields = nominalOrOrdinalFields.sort(compare);
                    // const f1 = sortedNominals[0];
                    var f2 = sortedFields[1];
                    if (f2.distinct) {
                        result = 1 / f2.distinct;
                        if (f2.distinct > 6 && chartType === 'heatmap') {
                            result = 2;
                        }
                        else if (chartType === 'heatmap') {
                            result = 0;
                        }
                    }
                }
            }
            return result;
        }),
    ];
};
