"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataToDataProps = void 0;
var tslib_1 = require("tslib");
var DWAnalyzer = tslib_1.__importStar(require("@antv/dw-analyzer"));
/**
 * Return Data Properties of dataset.
 *
 * @public
 * @param data - dataset
 * @param fields - selected fields
 */
function dataToDataProps(data, fields) {
    if (!data) {
        throw new Error('Argument `data` is missing.');
    }
    var dataTypeInfos = DWAnalyzer.typeAll(data, fields);
    var dataProps = [];
    dataTypeInfos.forEach(function (info) {
        var lom = [];
        if (DWAnalyzer.isNominal(info))
            lom.push('Nominal');
        if (DWAnalyzer.isOrdinal(info))
            lom.push('Ordinal');
        if (DWAnalyzer.isInterval(info))
            lom.push('Interval');
        if (DWAnalyzer.isDiscrete(info))
            lom.push('Discrete');
        if (DWAnalyzer.isContinuous(info))
            lom.push('Continuous');
        if (DWAnalyzer.isTime(info))
            lom.push('Time');
        var newInfo = tslib_1.__assign(tslib_1.__assign({}, info), { levelOfMeasurements: lom });
        dataProps.push(newInfo);
    });
    return dataProps;
}
exports.dataToDataProps = dataToDataProps;
