"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dataPropsToAdvices = void 0;
var tslib_1 = require("tslib");
var knowledge_1 = require("@antv/knowledge");
var util_1 = require("@antv/util");
var rules_1 = require("../rules");
var get_1 = tslib_1.__importDefault(require("lodash/get"));
var spec_mapping_1 = require("./spec-mapping");
var custom_plot_1 = require("../custom-plot");
function scoreRules(chartType, dataProps, options) {
    var showLog = options === null || options === void 0 ? void 0 : options.showLog;
    var purpose = options ? options.purpose : '';
    var preferences = options ? options.preferences : undefined;
    var chartRuleConfigs = (options === null || options === void 0 ? void 0 : options.chartRuleConfigs) || {};
    var ChartRules = rules_1.initChartRules(options === null || options === void 0 ? void 0 : options.chartKnowledge);
    // for log
    var record = [];
    var hardScore = 1;
    ChartRules.filter(function (r) {
        return r.hardOrSoft === 'HARD' && r.specChartTypes.includes(chartType) && !get_1.default(chartRuleConfigs, r.id + ".off");
    }).forEach(function (hr) {
        var customConfigs = get_1.default(chartRuleConfigs, "" + hr.id) || {};
        var score = hr.check(tslib_1.__assign({ dataProps: dataProps, chartType: chartType, purpose: purpose, preferences: preferences }, customConfigs));
        hardScore *= score;
        record.push({ name: hr.id, score: score, hard: true });
    });
    /**
     * TODO
     * new soft rules score method
     */
    var softScore = 0;
    // let softFullScore = 0;
    ChartRules.filter(function (r) {
        return r.hardOrSoft === 'SOFT' && r.specChartTypes.includes(chartType) && !get_1.default(chartRuleConfigs, r.id + ".off");
    }).forEach(function (sr) {
        var customConfigs = get_1.default(chartRuleConfigs, "" + sr.id) || {};
        // const weight = _get(chartRuleConfigs, `${sr.id}.weight`) || sr.weight;
        // softFullScore += weight * 1;
        var score = sr.check(tslib_1.__assign({ dataProps: dataProps, chartType: chartType, purpose: purpose, preferences: preferences }, customConfigs));
        softScore += score;
        record.push({ name: sr.id, score: score, hard: false });
    });
    // const score = hardScore * 100 * (softFullScore ? softScore / softFullScore : 0);
    var score = hardScore * (1 + softScore);
    if (showLog)
        console.log('💯score: ', score, '=', hardScore, '* (1 +', softScore, ') ;charttype: ', chartType);
    if (showLog)
        console.log(record);
    return score;
}
function applyDesignRules(chartType, dataProps, chartTypeSpec) {
    var toCheckRules = rules_1.DesignRules.filter(function (rule) { return rule.domain.indexOf(chartType) !== -1; });
    var encodingSpec = toCheckRules.reduce(function (lastSpec, rule) {
        var relatedSpec = rule.optimizer(dataProps, chartTypeSpec);
        return util_1.deepMix(lastSpec, relatedSpec);
    }, {});
    return encodingSpec;
}
/**
 * @public
 */
function dataPropsToAdvices(dataProps, options, dataset) {
    var enableRefine = (options === null || options === void 0 ? void 0 : options.refine) === undefined ? true : options.refine;
    var showLog = options === null || options === void 0 ? void 0 : options.showLog;
    var ChartWIKI = (options === null || options === void 0 ? void 0 : options.chartKnowledge) ? options.chartKnowledge : knowledge_1.CKBJson('en-US', true);
    var CHART_ID_OPTIONS = Object.keys(ChartWIKI);
    // score every
    var list = CHART_ID_OPTIONS.map(function (t) {
        // step 1: analyze score by rule
        var score = scoreRules(t, dataProps, options);
        if (score <= 0)
            return { type: t, spec: null, score: score };
        // step 2: field mapping to spec encoding
        var chartTypeSpec = spec_mapping_1.getChartTypeSpec(t, dataProps, dataset);
        // FIXME kpi_panel and table spec 暂时可以为 null, 之后随需求增加 cfg
        if (!custom_plot_1.customChartType.includes(t) && !chartTypeSpec)
            return { type: t, spec: null, score: 0 };
        // step 3: apply design rules
        if (chartTypeSpec && enableRefine) {
            var encodingSpecs = applyDesignRules(t, dataProps, chartTypeSpec);
            util_1.deepMix(chartTypeSpec.encoding, encodingSpecs);
            // return { type: t, spec: chartTypeSpec, score }
        }
        return { type: t, spec: chartTypeSpec, score: score };
    });
    function compareAdvices(chart1, chart2) {
        if (chart1.score < chart2.score) {
            return 1;
        }
        else if (chart1.score > chart2.score) {
            return -1;
        }
        else {
            return 0;
        }
    }
    // filter and sorter
    var resultList = list.filter(function (e) { return e.score > 0; }).sort(compareAdvices);
    if (showLog)
        console.log('🍒🍒🍒🍒🍒🍒 resultList 🍒🍒🍒🍒🍒🍒');
    if (showLog)
        console.log(resultList);
    return resultList;
}
exports.dataPropsToAdvices = dataPropsToAdvices;
