import { LevelOfMeasurement as LOM, ChartID, Purpose, ChartKnowledgeJSON } from '@antv/knowledge';
import * as DWAnalyzer from '@antv/dw-analyzer';
import { Preferences, ChartRuleConfigMap } from '../rules';
import { Mark, EncodingType, EncodingKey, Aggregation, StackType } from './vega-lite';
/**
 * @public
 */
export interface AdvisorOptions {
    /**
     * 分析目的 对应 ckb purpose
     * Comparison -- 比较
     * Trend -- 趋势
     * Distribution -- 分布
     * Rank -- 排行
     * Proportion -- 比例
     * Composition -- 组成
     */
    purpose?: Purpose;
    /**
     * 偏好设置
     */
    preferences?: Preferences;
    /**
     * 是否应用设计规则
     */
    refine?: boolean;
    /**
     * 自定义调整规则
     */
    chartRuleConfigs?: ChartRuleConfigMap;
    /**
     * custom charts knowledge
     */
    chartKnowledge?: Record<string, ChartKnowledgeJSON>;
    /**
     * specify fields in the dataset
     */
    fields?: string[];
    /**
     * log on/off
     */
    showLog?: boolean;
}
/**
 * return type of data to data props, describe data column
 * @public
 */
export declare type DataProperty = (DWAnalyzer.NumberFieldInfo & {
    name: string;
    levelOfMeasurements: LOM[];
}) | (DWAnalyzer.DateFieldInfo & {
    name: string;
    levelOfMeasurements: LOM[];
}) | (DWAnalyzer.StringFieldInfo & {
    name: string;
    levelOfMeasurements: LOM[];
});
/**
 * @public
 */
export declare type VegaLiteEncodingSpecification = Partial<Record<EncodingKey, {
    field?: string;
    type?: EncodingType;
    bin?: boolean;
    aggregate?: Aggregation;
    stack?: StackType;
    scale?: any;
    domain?: any;
    ticks?: any;
}>>;
/**
 * @public
 */
export interface SingleViewSpec {
    mark: {
        type: Mark;
        [record: string]: any;
    };
    encoding: VegaLiteEncodingSpecification;
}
/**
 * @public
 */
export declare type VegaLiteSubsetSpec = SingleViewSpec | {
    layer: SingleViewSpec[];
};
/**
 * @public
 */
export declare type Specification = SingleViewSpec;
/**
 * @public
 */
export interface TableDataCfg {
    rows: string[];
    values: string[];
    columns: string[];
}
/**
 * return type of data props to spec
 * @public
 */
export interface Advice {
    type: ChartID;
    spec: Specification | TableDataCfg | null;
    score: number;
}
/**
 * @public
 */
export declare type ChartLibrary = 'G2Plot' | 'G2';
/**
 * @public
 */
export declare type G2PlotChartType = 'Line' | 'Area' | 'Column' | 'Bar' | 'Pie' | 'Rose' | 'Scatter' | 'Histogram' | 'Heatmap';
/**
 * @public
 */
export interface G2PlotConfig {
    type: G2PlotChartType;
    configs: Record<string, any>;
}
