"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.G2PLOT_TYPE_MAPPING = exports.adviceToLibConfig = void 0;
var lodash_1 = require("lodash");
var custom_plot_1 = require("../custom-plot");
// export function adviceToLibConfig(advice: Advice, libraryName: 'ECharts'): EChartsConfig | null;
function adviceToLibConfig(advice, libraryName) {
    if (libraryName === void 0) { libraryName = 'G2Plot'; }
    switch (libraryName) {
        case 'G2':
        case 'G2Plot':
            return g2plotAdaptor(advice);
        // case 'echarts':
        //   return echartsAdaptor(advice);
        default:
            return g2plotAdaptor(advice);
    }
}
exports.adviceToLibConfig = adviceToLibConfig;
/**
 * @public
 */
exports.G2PLOT_TYPE_MAPPING = {
    line_chart: 'Line',
    area_chart: 'Area',
    stacked_area_chart: 'Area',
    percent_stacked_area_chart: 'Area',
    column_chart: 'Column',
    grouped_column_chart: 'Column',
    stacked_column_chart: 'Column',
    percent_stacked_column_chart: 'Column',
    bar_chart: 'Bar',
    grouped_bar_chart: 'Bar',
    stacked_bar_chart: 'Bar',
    percent_stacked_bar_chart: 'Bar',
    histogram: 'Histogram',
    pie_chart: 'Pie',
    donut_chart: 'Pie',
    rose_chart: 'Rose',
    scatter_plot: 'Scatter',
    bubble_chart: 'Scatter',
    heatmap: 'Heatmap',
};
var G2PLOT_ENCODING_MAPPING = {
    'x.ticks': 'xAxis.tickLine',
    'x.domain': 'xAxis.line',
    'y.scale.domainMin': 'yAxis.min',
};
function g2plotAdaptor(advice) {
    var _a, _b, _c;
    var type = advice.type, spec = advice.spec;
    // kpi panel and table can not be render by g2plot, it will have custom config
    if (custom_plot_1.customChartType.includes(type))
        return null;
    var chartType = exports.G2PLOT_TYPE_MAPPING[type];
    if (!chartType || !spec)
        return null;
    // TODO 暂时没有多layer的输出，先粗暴处理
    if (spec.layer)
        return null;
    var configs = {};
    // the rest scene is single view, transfer to  g2plot config
    var encoding = spec.encoding;
    // step1: common config decode
    var CHANNEL_MAPPING = {
        x: 'xField',
        y: 'yField',
        theta: 'angleField',
        color: 'colorField',
        size: 'sizeField',
    };
    Object.keys(encoding).forEach(function (e) {
        var key = CHANNEL_MAPPING[e];
        var channel = encoding[e];
        if (key && channel && channel.field) {
            configs[key] = channel.field;
        }
    });
    Object.keys(G2PLOT_ENCODING_MAPPING).forEach(function (key) {
        if (lodash_1.has(encoding, key)) {
            lodash_1.set(configs, G2PLOT_ENCODING_MAPPING[key], lodash_1.get(encoding, key));
        }
    });
    // step2: map color to series
    var seriesCharts = [
        'line_chart',
        'stacked_area_chart',
        'percent_stacked_area_chart',
        'grouped_column_chart',
        'stacked_column_chart',
        'percent_stacked_column_chart',
        'grouped_bar_chart',
        'stacked_bar_chart',
        'percent_stacked_bar_chart',
    ];
    if (seriesCharts.includes(type)) {
        configs.seriesField = configs.colorField;
    }
    // step3: add isStack, isGroup, isPercent
    var stackCharts = [
        'stacked_column_chart',
        'percent_stacked_column_chart',
        'stacked_bar_chart',
        'percent_stacked_bar_chart',
    ];
    if (stackCharts.includes(type)) {
        configs.isStack = true;
    }
    var groupCharts = ['grouped_bar_chart', 'grouped_column_chart'];
    if (groupCharts.includes(type)) {
        configs.isGroup = true;
    }
    var percentCharts = [
        'percent_stacked_column_chart',
        'percent_stacked_bar_chart',
        // TODO 目前 g2plot 没有直接支持百分比堆叠面积图，之后直接升级即可
        'percent_stacked_area_chart',
    ];
    if (percentCharts.includes(type)) {
        configs.isPercent = true;
    }
    // step4: special config
    if (type === 'grouped_column_chart') {
        configs.xField = (_a = encoding.column) === null || _a === void 0 ? void 0 : _a.field;
    }
    if (type === 'grouped_bar_chart') {
        configs.yField = (_b = encoding.row) === null || _b === void 0 ? void 0 : _b.field;
    }
    if (type === 'histogram') {
        configs.binField = (_c = encoding.x) === null || _c === void 0 ? void 0 : _c.field;
    }
    if (type === 'donut_chart') {
        configs.innerRadius = 0.6;
    }
    if (type === 'bubble_chart') {
        // FIXME g2plot 目前必须指定散点图的 size 已反馈给 g2 之后会做统一处理
        configs.size = [2, 10];
        configs.shape = 'circle';
    }
    return { type: chartType, configs: configs };
}
// function echartsAdaptor(advice: Advice): EChartsConfig {}
