import { AutoPlot } from './auto-plot';
import { DummyPlot } from './dummy-plot';
import { AdvisorOptions, G2PlotConfig } from '../advice-pipeline';
import { Preferences, ChartRuleConfigMap } from '../rules';
export { Preferences };
import { Language } from '../i18n';
/**
 * autochart configs
 * @public
 */
export interface AutoChartOptions extends AdvisorOptions {
    /**
     * The chart title defines text to draw at the top of the chart.
     */
    title?: string;
    /**
     * The chart's caption, which will render above the chart.
     */
    description?: string;
    /**
     * data columns(fields)
     */
    fields?: string[];
    /**
     * chart type switch tool
     * */
    toolbar?: boolean;
    /**
     * dev mode
     * */
    development?: boolean;
    /**
     * chart theme, including 'dark' and 'light'
     */
    theme?: string;
    /**
     * g2plot configs
     */
    config?: G2PlotConfig;
    /**
     * i18n settings, support English(en-US) and Chinese(zh-CN).
     */
    language?: Language;
    /**
     * render while no data
     */
    noDataContent?: (container: HTMLDivElement) => void;
    /** render while no recommend result */
    feedback?: (container: HTMLDivElement) => void;
}
export { AdvisorOptions, ChartRuleConfigMap };
export declare function addCanvas(layer: HTMLElement, { title, description }: AutoChartOptions): HTMLDivElement;
/**
 * @public
 */
export declare class AutoChart {
    static create(container: HTMLElement | string, data: any[] | Promise<any[]>, options?: AutoChartOptions): Promise<AutoChart>;
    private container;
    private rendered;
    private noDataLayer;
    private canvasLayer;
    constructor(container: HTMLElement);
    private development;
    private data;
    private options;
    private toolbar?;
    private configPanel?;
    private plot?;
    private noDataContent;
    private mockPanel?;
    private isMocked;
    setup(data: any[], options?: AutoChartOptions): Promise<this>;
    render(): Promise<void>;
    /**
     * 清除上次渲染遗留的元素
     */
    destroy(): void;
    /**
     * get plot detail
     */
    getPlot(): AutoPlot | DummyPlot | undefined;
}
