"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * 让元素可以拖动改变位置
 * @param target - 需要拖动的元素
 */
function draggable(target) {
    var handlers = [];
    target.onmousedown = function (event) {
        var shiftX = event.clientX - target.getBoundingClientRect().left;
        var shiftY = event.clientY - target.getBoundingClientRect().top;
        var mask = target.querySelector('[data-id="mask"]');
        function moveAt(pageX, pageY) {
            target.style.left = pageX - shiftX + 'px';
            target.style.top = pageY - shiftY + 'px';
        }
        moveAt(event.pageX, event.pageY);
        function onMouseMove(event) {
            mask.style.zIndex = '10';
            moveAt(event.pageX, event.pageY);
        }
        handlers.push(onMouseMove);
        // move the ball on mousemove
        document.addEventListener('mousemove', onMouseMove);
        // drop the ball, remove unneeded handlers
        target.onmouseup = function () {
            handlers.forEach(function (item) {
                document.removeEventListener('mousemove', item);
            });
            mask.style.zIndex = '-1';
            target.onmouseup = null;
        };
    };
    target.ondragstart = function () {
        return false;
    };
}
exports.default = draggable;
