"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Toolbar = void 0;
var tslib_1 = require("tslib");
var thumbnails_1 = tslib_1.__importDefault(require("@antv/thumbnails"));
var knowledge_1 = require("@antv/knowledge");
var style_1 = require("../style");
var advice_pipeline_1 = require("../advice-pipeline");
var util_1 = require("../util");
var custom_plot_1 = require("../custom-plot");
var i18n_1 = require("../i18n");
function getThumbnailURL(chartId) {
    var _a, _b;
    if ((_a = thumbnails_1.default[chartId]) === null || _a === void 0 ? void 0 : _a.svgCode) {
        return "data:image/svg+xml;utf8," + encodeURIComponent((_b = thumbnails_1.default[chartId]) === null || _b === void 0 ? void 0 : _b.svgCode);
    }
    return 'https://gw.alipayobjects.com/zos/antfincdn/lP6YFnCEjy/nochartimg.svg';
}
var rankIcons = [
    'https://gw.alipayobjects.com/zos/antfincdn/61FtDvdTVl/no1.png',
    'https://gw.alipayobjects.com/zos/antfincdn/Y7AsvjRWNF/no2.png',
    'https://gw.alipayobjects.com/zos/antfincdn/2%24ruKwktmY/no3.png',
];
function getAdvicesHtml(advices) {
    var ChartWiki = knowledge_1.CKBJson(i18n_1.getLanguage(), true);
    var top3 = advices
        // TODO 暂时通过 filter 处理没有输出 g2plot lib config 的图表类型
        .filter(function (advice) { return custom_plot_1.customChartType.includes(advice.type) || advice_pipeline_1.adviceToLibConfig(advice); })
        .slice(0, 3);
    var rankContent = top3
        .map(function (item, i) {
        return "<div class=\"" + style_1.CLASS_PREFIX + "advice\" data-index=\"" + i + "\">\n        <div class=\"" + style_1.CLASS_PREFIX + "advice-thumbnail\" data-index=\"" + i + "\">\n          <img src=\"" + getThumbnailURL(item.type) + "\" data-index=\"" + i + "\" />\n        </div>\n        <div class=\"" + style_1.CLASS_PREFIX + "advice-desc\" data-index=\"" + i + "\">\n          <img src=\"" + rankIcons[i] + "\" data-index=\"" + i + "\"/>\n          <div class=\"advice-chart-name\" data-index=\"" + i + "\" style=\"font-size: " + (i18n_1.getLanguage() === 'zh-CN' ? '16px' : '12px') + "\" >" + ChartWiki[item.type].name + "</div>\n          <div class=\"advice-score-text\" data-index=\"" + i + "\" >" + i18n_1.intl.get('Score') + " <span class=\"advice-score\">" + item.score.toFixed(2) + "</span></div>\n        </div>\n      </div>";
    })
        .join('');
    return "\n    <div class=\"" + style_1.CLASS_PREFIX + "advice_content\">\n    " + rankContent + "\n      </div>\n    </div>\n    <div class=\"" + style_1.CLASS_PREFIX + "advice_content_arrow\">\n  ";
}
var TOOLBAR_HTML = "\n<div data-id=\"chart-type-btn\" class=\"" + style_1.CLASS_PREFIX + "config_btn\">\n  <img src=\"https://gw.alipayobjects.com/zos/antfincdn/krFnwF2VZi/retweet.png\" />\n</div>\n<div data-id=\"advices\" class=\"" + style_1.CLASS_PREFIX + "advice_container\"></div>\n";
var Toolbar = /** @class */ (function () {
    function Toolbar(plotInst, container) {
        var _this = this;
        this.mouseEnterHandler = function () {
            _this.toolbar.style.display = 'block';
        };
        this.mouseLeaveHandler = function () {
            _this.advicesContainer.style.display = 'none';
            _this.toolbar.style.display = 'none';
        };
        this.destroyed = false;
        this.plotInst = plotInst;
        this.chartContainer = container;
        container.addEventListener('mouseenter', this.mouseEnterHandler);
        container.addEventListener('mouseleave', this.mouseLeaveHandler);
        var toolbar = document.createElement('div');
        this.toolbar = toolbar;
        toolbar.className = style_1.CLASS_PREFIX + "toolbar";
        toolbar.innerHTML = TOOLBAR_HTML;
        this.advicesContainer = toolbar.querySelector('[data-id="advices"]');
        // 绑定事件
        toolbar.addEventListener('click', function (e) {
            var advicesContainer = _this.advicesContainer;
            if (e && e.target) {
                var target = e.target;
                if (target.getAttribute('data-id') === 'chart-type-btn') {
                    advicesContainer.style.display = util_1.getElementDisplay(advicesContainer) === 'none' ? 'block' : 'none';
                }
                else {
                    var dataIndex = target.getAttribute('data-index');
                    if (dataIndex) {
                        _this.plotInst.render(Number.parseInt(dataIndex, 10));
                    }
                }
            }
        });
        this.render();
    }
    Toolbar.prototype.render = function () {
        var advices = this.plotInst.advices;
        this.advicesContainer.innerHTML = getAdvicesHtml(advices);
        this.chartContainer.appendChild(this.toolbar);
    };
    Toolbar.prototype.destroy = function () {
        var _a = this, chartContainer = _a.chartContainer, mouseEnterHandler = _a.mouseEnterHandler, mouseLeaveHandler = _a.mouseLeaveHandler, toolbar = _a.toolbar, destroyed = _a.destroyed;
        if (destroyed)
            return;
        chartContainer.removeEventListener('mouseenter', mouseEnterHandler);
        chartContainer.removeEventListener('mouseleave', mouseLeaveHandler);
        chartContainer.removeChild(toolbar);
        this.destroyed = true;
    };
    return Toolbar;
}());
exports.Toolbar = Toolbar;
