"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.overallTrendsIW = void 0;
var utils_1 = require("./utils");
exports.overallTrendsIW = function (data) {
    var THRESHOLD = 0.85;
    var insights = [];
    var _a = utils_1.rowDataToColumnFrame(data), columnProps = _a.columnProps, columns = _a.columns;
    for (var i = 0; i < columns.length; i++) {
        if (columnProps[i].isTime) {
            var _loop_1 = function (j) {
                if (columnProps[j].isInterval) {
                    var couples = columns[i].map(function (value, index) {
                        return {
                            valueI: value,
                            valueJ: columns[j][index],
                        };
                    });
                    var sortedCouples = couples.sort(function (a, b) {
                        var dateA = Date.parse(a.valueI);
                        var dateB = Date.parse(b.valueI);
                        if (dateA < dateB) {
                            return -1;
                        }
                        if (dateA > dateB) {
                            return 1;
                        }
                        return 0;
                    });
                    var orderField_1 = [];
                    var valueField_1 = [];
                    sortedCouples.forEach(function (couple, index) {
                        orderField_1.push(index);
                        valueField_1.push(couple.valueJ);
                    });
                    var orderFieldNor = utils_1.normalizeArray(orderField_1);
                    var valueFieldNor = utils_1.normalizeArray(valueField_1);
                    var p = utils_1.pearsonCorr(orderFieldNor, valueFieldNor);
                    if (Math.abs(p) > THRESHOLD) {
                        insights.push({
                            type: 'OverallTrends',
                            description: "'" + columnProps[j].title + "' is trending " + (p > 0 ? 'upwards' : 'downwards') + " with '" + columnProps[i].title + "'",
                            fields: [columnProps[i].title, columnProps[j].title],
                            present: {
                                purpose: ['Trend'],
                                type: 'line_chart',
                                data: data,
                                encoding: {
                                    x: columnProps[i].title,
                                    y: columnProps[j].title,
                                },
                                configs: { xAxis: { title: { visible: true } }, yAxis: { title: { visible: true } } },
                            },
                        });
                    }
                }
            };
            for (var j = 0; j < columns.length; j++) {
                _loop_1(j);
            }
        }
    }
    return insights;
};
