import { LevelOfMeasurement as LOM, ChartID as ChartType } from '@antv/knowledge';
import { FieldInfo } from '@antv/dw-analyzer';
/**
 * @public
 */
export declare type HardOrSoft = 'HARD' | 'SOFT';
/**
 * this minimum subset of pipeline/DataProperty, make pipeline can start with step2(dataPropsToAdvice)
 * @public
 */
export interface BasicDataPropertyForAdvice {
    /** field name */
    readonly name: string;
    /** LOM */
    readonly levelOfMeasurements: LOM[];
    /** used for split column xy series */
    readonly samples: any[];
    /** required types in analyzer FieldInfo */
    readonly recommendation: FieldInfo['recommendation'];
    readonly type: FieldInfo['type'];
    readonly distinct?: number;
    readonly count?: number;
    readonly sum?: number;
    readonly maximum?: any;
    readonly minimum?: any;
    readonly missing?: number;
}
/**
 * 偏好选项
 * @public
 */
export interface Preferences {
    canvasLayout: 'landscape' | 'portrait';
}
/**
 * @public
 */
export interface Info {
    chartType: ChartType;
    dataProps: BasicDataPropertyForAdvice[];
    purpose?: string;
    preferences?: Preferences;
    customWeight?: number;
    [key: string]: any;
}
/**
 * @public
 */
export declare type Validator = (args: Info) => number;
/**
 * @public
 */
export declare const CHART_RULE_ID: readonly ["data-check", "data-field-qty", "no-redundant-field", "purpose-check", "series-qty-limit", "bar-series-qty", "line-field-time-ordinal", "landscape-or-portrait", "diff-pie-sector", "nominal-enum-combinatorial", "limit-series", "aggregation-single-row", "all-can-be-table"];
/**
 * @public
 */
export declare type ChartRuleID = typeof CHART_RULE_ID[number];
/**
 * @public
 */
export declare class Rule {
    private _id;
    private _hardOrSoft;
    private _specChartTypes;
    private _weight;
    private validator;
    constructor(id: ChartRuleID, hardOrSoft: HardOrSoft, specChartTypes: ChartType[], weight: number, validator: Validator);
    get id(): string;
    get hardOrSoft(): HardOrSoft;
    get specChartTypes(): ChartType[];
    get weight(): number;
    toString(): string;
    check(args: Info): number;
}
